package fr.cnrs.mri.util.tests;


import static org.junit.Assert.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.zip.CRC32;
import java.util.zip.Deflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import fr.cnrs.mri.files.ChecksumCalculator;
import fr.cnrs.mri.testData.TestImages;
import fr.cnrs.mri.util.ZIP32Util;
import fr.cnrs.mri.util.ZipEntryWithPath;

public class ZIPUtilTest {

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void zipTest() throws IOException {
		OutputStream outputStream = new FileOutputStream(new File("test.zip"));
		
		Integer compressionLevel = Deflater.BEST_COMPRESSION;
		
		ZipEntryWithPath firstEntry = new ZipEntryWithPath("fichier1.tif", TestImages.image01Head());
		ZipEntryWithPath secondEntry = new ZipEntryWithPath("fichier2.jpg", TestImages.jpg01());
		
		ArrayList<ZipEntryWithPath> entries = new ArrayList<ZipEntryWithPath>();
		entries.add(firstEntry);
		entries.add(secondEntry);
		
		ZIP32Util zipUtil = new ZIP32Util(outputStream, entries);
		zipUtil.setCompressionLevel(compressionLevel);
		
		zipUtil.zipFilesAndWriteToStream();
		
		outputStream.close();
		
		assertTrue(new File("test.zip").exists());
		
		ZipInputStream zipInput = new ZipInputStream(new FileInputStream(new File("test.zip")));
		ZipEntry file1 = zipInput.getNextEntry();
		zipInput.closeEntry();
		assertEquals("fichier1.tif", file1.getName());
		assertEquals(new File(TestImages.image01Head()).length(), file1.getSize());
		
		assertTrue(file1.getSize() > 0);
		assertTrue(file1.getCompressedSize() > 0);
		assertTrue(file1.getCompressedSize() < file1.getSize());
		
		ZipEntry file2 = zipInput.getNextEntry();
		zipInput.closeEntry();
		assertEquals("fichier2.jpg", file2.getName());
		
		assertTrue(file2.getSize() > 0);
		assertTrue(file2.getCompressedSize() > 0);
		assertTrue(file2.getCompressedSize() < file2.getSize());
		
		zipInput.close();
		new File("test.zip").delete();
		
	}
	
	@Test
	public void zipNoCompressionTest() throws IOException {
		OutputStream outputStream = new FileOutputStream(new File("test.zip"));
		
		Integer compressionLevel = Deflater.NO_COMPRESSION;
		
		ZipEntryWithPath firstEntry = new ZipEntryWithPath("fichier1.tif", TestImages.image01Head());
		ZipEntryWithPath secondEntry = new ZipEntryWithPath("fichier2.jpg", TestImages.jpg01());
		
		File firstFile = new File(TestImages.image01Head());
		File secondFile = new File(TestImages.jpg01());
		
		firstEntry.setSize(firstFile.length());
		firstEntry.setCompressedSize(firstFile.length());
		firstEntry.setMethod(ZipEntry.STORED);
		ChecksumCalculator csc = new ChecksumCalculator(firstFile.getAbsolutePath());
		csc.setAlgorithm(new CRC32());
		firstEntry.setCrc(Long.parseLong(csc.getChecksum(), 16));
		
		secondEntry.setSize(secondFile.length());
		secondEntry.setCompressedSize(secondFile.length());
		secondEntry.setMethod(ZipEntry.STORED);
		csc = new ChecksumCalculator(secondFile.getAbsolutePath());
		csc.setAlgorithm(new CRC32());
		secondEntry.setCrc(Long.parseLong(csc.getChecksum(), 16));
		
		ArrayList<ZipEntryWithPath> entries = new ArrayList<ZipEntryWithPath>();
		entries.add(firstEntry);
		entries.add(secondEntry);
		
		ZIP32Util zipUtil = new ZIP32Util(outputStream, entries);
		zipUtil.setCompressionLevel(compressionLevel);
		
		zipUtil.zipFilesAndWriteToStream();
		
		outputStream.close();
		
		assertTrue(new File("test.zip").exists());
		
		ZipInputStream zipInput = new ZipInputStream(new FileInputStream(new File("test.zip")));
		ZipEntry file1 = zipInput.getNextEntry();
		zipInput.closeEntry();
		assertEquals("fichier1.tif", file1.getName());
		assertEquals(new File(TestImages.image01Head()).length(), file1.getSize());
		
		assertTrue(file1.getSize() > 0);
		assertTrue(file1.getCompressedSize() > 0);
		assertTrue(file1.getCompressedSize() == file1.getSize());
		
		ZipEntry file2 = zipInput.getNextEntry();
		zipInput.closeEntry();
		assertEquals("fichier2.jpg", file2.getName());
		
		assertTrue(file2.getSize() > 0);
		assertTrue(file2.getCompressedSize() > 0);
		assertTrue(file2.getCompressedSize() == file2.getSize());
		
		zipInput.close();
		new File("test.zip").delete();
	}

	
	
	
	
	@Test
	public void testZipFolderStructure() throws IOException {
		OutputStream outputStream = new FileOutputStream(new File("test.zip"));
		
		Integer compressionLevel = Deflater.BEST_COMPRESSION;
		
		
		ZipEntryWithPath firstEntry = new ZipEntryWithPath("toto/fichier1.tif", TestImages.image01Head());
		ZipEntryWithPath secondEntry = new ZipEntryWithPath("fichier2.jpg", TestImages.jpg01());
		
		ArrayList<ZipEntryWithPath> entries = new ArrayList<ZipEntryWithPath>();
		entries.add(firstEntry);
		entries.add(secondEntry);
		
		ZIP32Util zipUtil = new ZIP32Util(outputStream, entries);
		zipUtil.setCompressionLevel(compressionLevel);
		
		zipUtil.zipFilesAndWriteToStream();
		
		outputStream.close();
		
		assertTrue(new File("test.zip").exists());
		
		ZipInputStream zipInput = new ZipInputStream(new FileInputStream(new File("test.zip")));
		ZipEntry file1 = zipInput.getNextEntry();
		zipInput.closeEntry();
		assertEquals("toto/fichier1.tif", file1.getName());
		assertEquals(new File(TestImages.image01Head()).length(), file1.getSize());
		
		assertTrue(file1.getSize() > 0);
		assertTrue(file1.getCompressedSize() > 0);
		assertTrue(file1.getCompressedSize() < file1.getSize());
		
		ZipEntry file2 = zipInput.getNextEntry();
		zipInput.closeEntry();
		assertEquals("fichier2.jpg", file2.getName());
		
		assertTrue(file2.getSize() > 0);
		assertTrue(file2.getCompressedSize() > 0);
		assertTrue(file2.getCompressedSize() < file2.getSize());
		
		zipInput.close();
		new File("test.zip").delete();
	}
	
	@Test
	public void zipBigFile() throws IOException {
		FileInputStream in = new FileInputStream("/home/alex/file-source/big.ndpi");
		ZipOutputStream out = new ZipOutputStream(new FileOutputStream("/home/alex/file-source/ndpi.zip"));
		out.putNextEntry(new ZipEntry("big.ndpi"));
		byte[] b = new byte[1024*1024*30];
		int count ;
		int i = 0;
		while ((count = in.read(b)) > 0) {
			System.out.print('*');
			if (i++ % 100 == 0) System.out.println();
			out.write(b, 0, count);
		}
		out.close();
		in.close();
	}
}
